using System;
using System.Diagnostics;
using System.Globalization;
using System.Windows.Input;
using Cirrious.CrossCore;
using Cirrious.CrossCore.Platform;
using Cirrious.MvvmCross.Plugins.Location;
using Cirrious.MvvmCross.Plugins.Messenger;
using Cirrious.MvvmCross.ViewModels;
using Geo.Core.Services;

namespace Geo.Core.ViewModels
{
    public class FirstViewModel
        : MvxViewModel
    {
        private readonly IMvxGeoLocationWatcher watcher;
        private readonly IGeocodeService geocode;
        private readonly IMvxMessenger messenger;

        public FirstViewModel(IMvxGeoLocationWatcher watcher, IGeocodeService geocode, IMvxMessenger messenger)
        {
            this.geocode = geocode;
            this.watcher = watcher;
            this.messenger = messenger;
            ShowPositionCommand = new MvxCommand(showPosition);
        }

        public class PositionMessage : MvxMessage
        {
            public PositionMessage(object sender) : base(sender)
            {
            }

            public double Lat { get; set; }
            public double Lng { get; set; }
            public string SLat { get; set; }
            public string SLng { get; set; }
        }

        private void showPosition()
        {
            var locale = new CultureInfo("en-US") {NumberFormat = {NumberGroupSeparator = string.Empty}};
            var p = new PositionMessage(this)
                        {
                            Lat = Lat,
                            Lng = Lng,
                            SLat = Lat.ToString(locale),
                            SLng = Lng.ToString(locale)
                        };
           messenger.Publish(p); 
        }

        private void onLocation(MvxGeoLocation location)
        {
            Lat = location.Coordinates.Latitude;
            Lng = location.Coordinates.Longitude;
            Alt = location.Coordinates.Altitude ?? 0;
            GeoObject = null; 
            geocode.StartSearchAsync(lng, lat,
                result => GeoObject = result,
                ex => MvxTrace.Error(ex.Message));
        }

        private void OnError(MvxLocationError error)
        {
            Mvx.Error("Seen location error {0}", error.Code);
        }


        public ICommand ShowPositionCommand { get; private set; }
        
        public void Init()
        {
            watcher.Start(new MvxGeoLocationOptions(), onLocation, OnError);
        }

        private RootObject geoObject;

        public RootObject GeoObject
        {
            get { return geoObject; }
            set
            {
                if (value == geoObject) return;
                geoObject = value;
                RaisePropertyChanged(() => GeoObject);
            }
        }

        private double lat;
        private const double Epsilon = 0.000001;

        public double Lat
        {
            get { return lat; }
            set
            {
                if (Math.Abs(value - lat) < Epsilon) return;
                lat = value;
                RaisePropertyChanged(() => Lat);
                RaisePropertyChanged(() => SLat);
            }
        }

        private double lng;

        public double Lng
        {
            get { return lng; }
            set
            {
                if (Math.Abs(value - lng) < Epsilon) return;
                lng = value;
                RaisePropertyChanged(() => Lng);
                RaisePropertyChanged(() => SLng);
            }
        }

        private double alt;

        public double Alt
        {
            get { return alt; }
            set
            {
                if (Math.Abs(value - alt) < Epsilon) return;
                alt = value;
                RaisePropertyChanged(() => Alt);
                RaisePropertyChanged(() => SAlt);
            }
        }

        public string SLat { get { return string.Format("{0,12:##0.00000000}",Lat); } }
        public string SLng { get { return string.Format("{0,12:##0.00000000}",Lng); } }
        public string SAlt { get { return string.Format("{0,12:##,##0.00000}",Alt); } }

    }
}
